void
on_mw_add_fd_released( GtkButton *, gpointer);

void
on_srch_dlg_result_clist_select_row( GtkCList *, gint, gint, 
                                     GdkEvent *, gpointer);

void
on_srch_dlg_srch_button_clicked( GtkButton *, gpointer);

void
on_srch_dlg_result_entry_changed( GtkEditable *, gpointer);

void
on_srch_dlg_measure_combo_entry_changed( GtkEditable *, gpointer);

void
on_srch_dlg_ok_button_released( GtkButton *, gpointer);

void
on_srch_dlg_cancel_button_released( GtkButton *, gpointer);

void
on_mw_edit_released ( GtkButton *, gpointer);
